;****************************************************************************************
;*                           TEMPERATUR-ANZEIGE-INNEN+AUSSEN-MIN+MAX VON DL3UKH         *
;*                           ==================================================         *
;*                                                                                      *
;*--------------------------------------------------------------------------------------*
;* PORT-PIN-ZUORDNUNG:                                                                  *
;* RA0 -AUSGANG UND EINGANG FUER DS18B20, ALSO SENDEN+EMPFANGEN AN DS18B20--AUSSEN     *
;* RA1 -AUSGANG UND EINGANG FUER DS18B20, ALSO SENDEN+EMPFANGEN AN DS18B20--INNEN      *
;* RA2 -AUSGANG DATEN/BEFEHL LCD-MODUL                                                  *
;* RA3 -AUSGANG ENABLE LCD-MODUL                                                        *
;* RA4 -EINGANG FR TASTER                                                              *
;* RA5 -EINGANG UNBENUTZT                                                               *
;* RA6 =OZS2 16MHZ QUARZ                                                                *
;* RA7 =OZS1 16MHZ QUARZ                                                                *
;*                                                                                      *
;* RB0 -AUSGANG LCD-MODUL   ALLES DATENBUS                                              *
;* RB1 -AUSGANG LCD-MODUL                                                               *
;* RB2 -AUSGANG LCD-MODUL                                                               *
;* RB3 -AUSGANG LCD-MODUL                                                               *
;*                                                                                      *
;* RA5 GEHT NUR ALS EINGANG WEGEN DEM ICSP-PROGRAMMIEREN HIER TEMP-RESET                *
;*                                                                                      *
;*--------------------------------------------------------------------------------------*
;* PROGRAMMIEREN DES PIC16F628A WIE FOLGT:                                              *
;* CP KEIN HAKEN     CPD KEIN HAKEN                                                     *
;* LVP KEIN HAKEN    BOREN KEIN HAKEN                                                   *
;* MCLR ALS I/O      PWRTEN DISABLE                                                     *
;* WDTE KEIN HAKEN   OSC HS                                                             *
;*                                                                                      *
;* PROGRAMMIERMODUS:                                                                    *
;* INTERNAL 5V   INTERNAL 5V                                                            *
;* TRISTATE      HVP                                                                    *
;*--------------------------------------------------------------------------------------*
;*                                                                                      *
;* ALS PIC-IC WURDE GENOMMEN PIC16F628A                                                 *
;* ALS OSZILLATOR WURDE EIN QUARZ MIT 16,00MHZ GENOMMEN. (T=0.0625S ALSO DIE TAKTZEIT) *
;*                                   EIN BEFEHL HAT MEIST 4 TAKTE SIND 0,25SEC         *
;*                                                                                      *
;*--------------------------------------------------------------------------------------*
;*                           BEGINN: 02.02.12                                           *
;*                           ENDE  : 08.02.12                                           *
;*                         GETESTET: 08.02.12                                           *
;*                        GEAENDERT: 31.01.19                                           *
;****************************************************************************************

        LIST    P=16F628A       
        TITLE   "TEMPERATUR-REGLER"

        ERRORLEVEL -302
        INCLUDE <P16F628A.INC>
__CONFIG   _HS_OSC & _WDT_OFF & _PWRTE_OFF & _BODEN_ON & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF & _CP_OFF
;--------------------------------------------------------------------------
;REGISTER ZUORDNEN NACH DEM DIE FESTEN-REGISTER ZU ENDE SIND
;ES GEHT ALSO ERST AB 22 HEX LOS
;DIES IST ALSO DIE RAM-SPEICHER ZUORDNUNG

               
HELP1                   EQU 0X22       ;HILFSZELLE
HELP2                   EQU 0X23       ;HILFSZELLE
ZAE                     EQU 0X24       ;ZAEHLER

TEMP_LI                 EQU 0X25       ;LOW-BYTE DES GEMESSENEN TEMPERATURWERTES INNEN
TEMP_HI                 EQU 0X26       ;HIGH-BYTE DES GEMESSENEN TEMPERATURWERTES INNEN
TEMP_LA                 EQU 0X27       ;LOW-BYTE DES GEMESSENEN TEMPERATURWERTES AUSSEN
TEMP_HA                 EQU 0X28       ;HIGH-BYTE DES GEMESSENEN TEMPERATURWERTES AUSSEN

TEM                     EQU 0X29       ;HILFSZELLE FR DIE LCD-BCD-ANZEIGE
TEMH                    EQU 0X2A       ;HILFSZELLE FR DIE LCD-BCD-ANZEIGE NEU HINZU
ANZ_L                   EQU 0X2B       ;HILFSZELLE FR DIE LCD-BCD-ANZEIGE
ANZ_H                   EQU 0X2C       ;HILFSZELLE FR DIE LCD-BCD-ANZEIGE
VER                     EQU 0X2D       ;HILFSZELLE FR DIE LCD-BCD-ANZEIGE
MINUS                   EQU 0X2E       ;HILFSZELLE FR MINUS-ZEICHEN
MERK                    EQU 0X2F       ;HILFSZELLE FR NEGATIVE-ZEICHEN

IA                      EQU 0X30       ;ANZEIGE INNEN + AUSSEN AUSWHLEN
TEMP_L                  EQU 0X31       ;LOW-BYTE DES GEMESSENEN TEMPERATURWERTES AUSSEN
TEMP_H                  EQU 0X32       ;HIGH-BYTE DES GEMESSENEN TEMPERATURWERTES AUSSEN

ADR                     EQU 0X33       ;EEPROM-ADRESSE
TEMP_HSP                EQU 0X34       ;ABGESPEICHERTE TEMPERATUR HIGH
TEMP_LSP                EQU 0X35       ;ABGESPEICHERTE TEMPERATUR LOW

KO_ST                   EQU 0X36       ;ZAHL NACH DER KOMMASTELLE MERKEN NEU HINZU
SPRUNG                  EQU 0X37       ;MERKER FUER SPRUNG NEU HINZU
  
BIT                     EQU 0X38       ;LCD HILFSZELLE
MERKE                   EQU 0X39       ;LCD HILFSZELLE
RESE_ANF                EQU 0X3A       ;AM ANFANG 1X RESETEN
                
;--------------------------- VARIABLEN -------------------------------------
BIT7                    EQU     7       ; BIT 7
BIT6                    EQU     6       ; BIT 6
BIT5                    EQU     5       ; BIT 5
BIT4                    EQU     4       ; BIT 4
BIT3                    EQU     3       ; BIT 3 
BIT2                    EQU     2       ; BIT 2
BIT1                    EQU     1       ; BIT 1 
BIT0                    EQU     0       ; BIT 0

;======================= FUER DS18B20=============================
TXRXA                   EQU     0       ;AUSSEN RA0 WIRD ZUM SENDEN/EMPFANGEN VON DATEN AN DEN DS18B80 GENOMMEN (TXRXA-FUEHLER)
TXRXI                   EQU     1       ; INNEN RA1 WIRD ZUM SENDEN/EMPFANGEN VON DATEN AN DEN DS18B80 GENOMMEN (TXRXI-FUEHLER)
SCHALT                  EQU     4       ;PORTA BIT 4 TASTER ZUM ABFRAGEN DER MIN + MAX TEMPERATUREN / UND RESET
AUTORESET               EQU     5       ; RA5 ALS EINGANG FUER AUTOMATISCHES RESET ZUR UHRZEIT VON SCHALTUHR
;==================================================================

;======================= FUER LCD-MODUL ===========================
RS                      EQU     2       ; PORTA BIT2 UMSCHALTEN BEFEHL/DATEN      BEI WRITE DATEN IST ES HIGH
EN                      EQU     3       ; PORTA BIT3 ENABLE FALLENDE FLANKE
;==================================================================

;------------------------- MACROS -----------------------------------------

BANK0           MACRO                   ;MACRO TO SELECT DATA RAM BANK 0
                BCF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK1           MACRO                   ;MACRO TO SELECT DATA RAM BANK 1
                BSF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM

BANK2           MACRO                   ;MACRO TO SELECT DATA RAM BANK 2
                BCF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

BANK3           MACRO                   ;MACRO TO SELECT DATA RAM BANK 3
                BSF     STATUS,RP0
                BSF     STATUS,RP1
                ENDM

TXRX_OUTPUTI    MACRO
                BSF     STATUS,RP0              ;T=4    ;BANK1-UMSCHALTUNG
                BCF     TRISA, TXRXI            ;T=4    ;AM PORTA --> RA1 ALS AUSGANG
                BCF     STATUS,RP0              ;T=4    ;BANK0-UMSCHALTUNG
                BCF     PORTA, TXRXI            ;T=4
                ENDM                            ;GES.: 1SEC


TXRX_INPUTI     MACRO
                BSF     STATUS,RP0              ;T=4    ;BANK1-UMSCHALTUNG
                BSF     TRISA, TXRXI            ;T=4    ;AM PORTA --> RA1 ALS EINGANG
                BCF     STATUS,RP0              ;T=4    ;BANK0-UMSCHALTUNG
                ENDM                            ;GES.: 1SEC


TXRX_OUTPUTA    MACRO
                BSF     STATUS,RP0              ;T=4    ;BANK1-UMSCHALTUNG
                BCF     TRISA, TXRXA            ;T=4    ;AM PORTA --> RA0 ALS AUSGANG
                BCF     STATUS,RP0              ;T=4    ;BANK0-UMSCHALTUNG
                BCF     PORTA, TXRXA            ;T=4
                ENDM                            ;GES.: 1SEC


TXRX_INPUTA     MACRO
                BSF     STATUS,RP0              ;T=4    ;BANK1-UMSCHALTUNG
                BSF     TRISA, TXRXA            ;T=4    ;AM PORTA --> RA0 ALS EINGANG
                BCF     STATUS,RP0              ;T=4    ;BANK0-UMSCHALTUNG
                ENDM                            ;GES.: 1SEC

;-----------------------------------------------------------------------------
;SPEICHERANFANG BZW. BEGINN

                ORG 0X0000
                GOTO MAIN               ; SPRUNG ZUM HAUPTPROGRAMM
                ORG 0X0004
                RETFIE                  ; BEI EINEM INTERRUPT SOFORT EIN RETURN

                ORG 0X0010

;*****************************************************************************
;*                            HAUPTPROGRAMM                                  *
;*                              01.02.11                                     *
;*****************************************************************************
;PORT-INITIALISIERUNG
MAIN            CLRF    PORTA                           
                CLRF    PORTB
                MOVLW   0X07                    ;COMPARATORS ALLE AUS VOM PORT A
                MOVWF   CMCON
                BANK1
                MOVLW   B'11110000'             ;INIT INPUT/OUTPUT AM PORTA --> BEI 0 ALLES AUSGAENGE NUR RA0 + 1 / 4+5+6+7 EINGANG  6+7 SIND QUARZ
                MOVWF   TRISA
                MOVLW   B'00001111'             ;INIT I/O PORTB ALLES AUSGAENGE ABER RB0 BIS RB3 EINGANG DA 4-BIT-DATENBUS 
                MOVWF   TRISB

;INIT TIMER 0 FUER 1MSEC
                MOVLW   B'10000111'             ;BIT0-2 1=PRESCALER256, BIT3 0=TMR0, BIT4 0=ENTFAELLT, BIT5 0=TAKT VOM SYSTEM, BIT6 0=KEIN RB-INTERRUPT, BIT7 1=PULL-UPS DISABLE 
                MOVWF   OPTION_REG      
                BANK0

;GRUNDEINSTELLUNGEN ; ACHTUNG TXRXI +A  AN RA0+1 = GEHT ABER, DA BEIM ANFANG ALS EINGANG = SIEHE INITIALISIERUNG OBEN
                CLRF    PORTA

;TIMER1 FUER ANDERES NUTZEN
;INIT TIMER 1 
;                MOVLW   0X30                    ;PRESCALER 8X / TAKT 4X / OSZ-ENABLE-BIT3 ? JETZT HIGH/ TMR1ON IST AUS, ES ZAEHLT ALSO NICHT
;                MOVWF   T1CON

;50MSEC ZEITSCHLEIFE
                MOVLW   D'50'                 ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 50 ALSO---> 50*1MSEC=50MSEC
                CALL    ZS1MS                 ;ZEITSCHLEIFE ABARBEITEN

;INITIALISIERUNG DOT-MATRIX-LCD
                MOVLW   H'28'                   ;FUNCTION SET  4-BIT-LAENGE, 2/4-ZEILIG, 5X7FONT
                CALL    BEFEHL

                MOVLW   H'0C'                   ;DISPLAY ON/OFF   DISPLAY ON, CURSOR OFF, BLINK OFF
                CALL    BEFEHL

                MOVLW   H'01'                   ;CLEAR DISPLAY
                CALL    BEFEHL

;3MSEC ZEITSCHLEIFE
                MOVLW   D'3'                  ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 3 ALSO---> 3*1MSEC=3MSEC
                CALL    ZS1MS                 ;ZEITSCHLEIFE ABARBEITEN

;---------------INITIALISIERUNG--ZU--ENDE-----------------------
            
ANFANG1     CALL    GEHT                    ;LCD-DOT-MATRIX "GEHT GLEICH LOS" USW...
            BSF     RESE_ANF, BIT0          ;AM ANFANG 1X RESETEN
            
;3SEC ZEITSCHLEIFE
            MOVLW   D'30'                   ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 30 ALSO---> 30*100MSEC=3000MSEC=3SEC
            CALL    ZS100MS                 ;ZEITSCHLEIFE ABARBEITEN

ANFANG3     CALL    LCD_LOE                 ;LCD-INNEN-AUSSEN-TEMP... DARSTELLEN
            CALL    AUSSEN
            CALL    INNEN


ANFANG      CALL    RESETI
            CALL    RESETA                   

            MOVLW   0XCC                    ;ROM BRAUCHT JETZT NICHT GESENDET WERDEN
            CALL    TXI
            MOVLW   0X44                    ;AUFFORDERUNG ZUR TEMPERATUR-MESSUNG IM DS18S20
            CALL    TXI

            MOVLW   0XCC                    ;ROM BRAUCHT JETZT NICHT GESENDET WERDEN
            CALL    TXA
            MOVLW   0X44                    ;AUFFORDERUNG ZUR TEMPERATUR-MESSUNG IM DS18S20
            CALL    TXA

;TASTERABFRAGE + 1SEC ZEITSCHLEIFE
            MOVLW   D'10'
            MOVWF   HELP2
WIEDER1     MOVLW   D'100'                  ;DAMIT SIND 1MSEC * 100 = 100MSEC
            MOVWF   ZAE
WIEDER2     BTFSS   PORTA, SCHALT           ;ALLER 100MSEC
            GOTO    TASTER                  ;BEI TASTER LOW ERFOLGT SPRUNG ZUR MARKE: TASTER

            BTFSS   PORTA, AUTORESET        ;AUTORESET
            GOTO    AUTO

;1MSEC ZEITSCHLEIFE
            MOVLW   D'1'                    ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 1 ALSO---> 1*1MSEC=1MSEC
            CALL    ZS1MS                   ;ZEITSCHLEIFE ABARBEITEN
;ENDE 1MSEC ZEITSCHLEIFE
            DECFSZ  ZAE, F
            GOTO    WIEDER2
            DECFSZ  HELP2, F
            GOTO    WIEDER1
;ENDE --- TASTERABFRAGE + 1SEC ZEITSCHLEIFE

            CALL    RESETI
            CALL    RESETA

            MOVLW   0XCC                    ;ROM BRAUCHT JETZT NICHT GESENDET WERDEN
            CALL    TXI
            MOVLW   0XBE                    ;AUFFORDERUNG ZUR TEMPERATUR AUSGABE
            CALL    TXI                     ;NACH DIESEM BEFEHL MUESSEN ALSO DIE TEMPERATUR USW... SOFORT EMPFANGEN WERDEN

            MOVLW   0XCC                    ;ROM BRAUCHT JETZT NICHT GESENDET WERDEN
            CALL    TXA
            MOVLW   0XBE                    ;AUFFORDERUNG ZUR TEMPERATUR AUSGABE
            CALL    TXA                     ;NACH DIESEM BEFEHL MUESSEN ALSO DIE TEMPERATUR USW... SOFORT EMPFANGEN WERDEN

;TEMPERATUR AUSLESEN FR AUSSEN
            CALL    RXA                     ;EIN BYTE EMPFANGEN
            MOVWF   TEMP_LA                 ;ABSPEICHERN DES ERGEBNISBYTES IN TEMP_L
            CALL    RXA                     ;EIN BYTE EMPFANGEN
            MOVWF   TEMP_HA                 ;ABSPEICHERN DES ERGEBNISBYTES IN TEMP_H
;TEMPERATUR AUSLESEN FR INNEN
            CALL    RXI                     ;EIN BYTE EMPFANGEN
            MOVWF   TEMP_LI                 ;ABSPEICHERN DES ERGEBNISBYTES IN TEMP_L
            CALL    RXI                     ;EIN BYTE EMPFANGEN
            MOVWF   TEMP_HI                 ;ABSPEICHERN DES ERGEBNISBYTES IN TEMP_H

;TEMPERATUR ANZEIGEN FR INNEN
            MOVF    TEMP_LI, W
            MOVWF   TEMP_L
            MOVF    TEMP_HI, W
            MOVWF   TEMP_H
            BSF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE

;TEMPERATUR ANZEIGEN FR AUSSEN
            MOVF    TEMP_LA, W
            MOVWF   TEMP_L
            MOVF    TEMP_HA, W
            MOVWF   TEMP_H
            BCF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE
            
            CALL    VERGLEICH               ;MAX+MIN BERECHNEN UND ABSPEICHERN
            
;NACH DEM 1X MESSEN TEMPERATUR ALS RESET IN EEPROM EINTRAGEN
            BTFSS   RESE_ANF, BIT0          ;BEIM 1 MAL RESETEN
            GOTO    WIED
            
            BCF     RESE_ANF, BIT0
;RESET AUSFHREN
;AKTUELLE TEMPERATUREN BERNEHMEN IN ALLE SPEICHER
            MOVLW   0X00
            MOVWF   ADR                     ;EEPROMADRESSE IN ADR
            MOVF    TEMP_LA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_HA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_LA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_HA, W
            CALL    E_WRITE
            
;;; FR INNEN
            INCF    ADR, F
            MOVF    TEMP_LI, W
            CALL    E_WRITE
                    
            INCF    ADR, F
            MOVF    TEMP_HI,W
            CALL    E_WRITE

            INCF    ADR, F
            MOVF    TEMP_LI, W
            CALL    E_WRITE
                    
            INCF    ADR, F
            MOVF    TEMP_HI, W
            CALL    E_WRITE
            
WIED        
;TASTERABFRAGE + 30SEC ZEITSCHLEIFE
            MOVLW   D'300'
            MOVWF   HELP2
WIEDE1      MOVLW   D'150'                  ;DAMIT SIND 1MSEC * 150 = 150MSEC
            MOVWF   ZAE
WIEDE2      BTFSS   PORTA, SCHALT           ;ALLER 150MSEC
            GOTO    TASTER                  ;BEI TASTER LOW ERFOLGT SPRUNG ZUR MARKE: TASTER

            BTFSS   PORTA, AUTORESET        ;AUTORESET
            GOTO    AUTO

;1MSEC ZEITSCHLEIFE
            MOVLW   D'1'                    ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 1 ALSO---> 1*1MSEC=1MSEC
            CALL    ZS1MS                   ;ZEITSCHLEIFE ABARBEITEN
;ENDE 1MSEC ZEITSCHLEIFE
            DECFSZ  ZAE, F
            GOTO    WIEDE2
            DECFSZ  HELP2, F
            GOTO    WIEDE1
;ENDE --- TASTERABFRAGE + 30SEC ZEITSCHLEIFE

            GOTO    ANFANG

;-------------------------------------------------------------------



;ZEITSCHLEIFE 1,5SEC
;WENN DER TASTER LNGER ALS 1,5SEC GEDRCKT WURDE GEHT ES ZUM RESET-PROG.
;WENN ABER KRZER GEHT ES ZU MINMAX-PROG.
TASTER      MOVLW   D'15'
            MOVWF   ZAE
WIEDER3     BTFSC   PORTA, SCHALT           ;BEI IMMER NOCH GEDRCKTER TASTER (ALSO LOW) ZEITSCHLEIFE IMMER NOCH ABARBEITEN
            GOTO    MINMAX                  ;BEI HIGH -ALSO TASTER LOSGELASSEN  SPRUNG ZU MARKE MINMAX
            MOVLW   D'1'
            CALL    ZS100MS
            DECFSZ  ZAE, F
            GOTO    WIEDER3

AUTO        CALL    LCD_LOE
            CALL    RESETANZ
;RESET AUSFHREN
;AKTUELLE TEMPERATUREN BERNEHMEN IN ALLE SPEICHER
            MOVLW   0X00
            MOVWF   ADR                     ;EEPROMADRESSE IN ADR
            MOVF    TEMP_LA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_HA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_LA, W
            CALL    E_WRITE
            
            INCF    ADR, F
            MOVF    TEMP_HA, W
            CALL    E_WRITE
            
;;; FR INNEN
            INCF    ADR, F
            MOVF    TEMP_LI, W
            CALL    E_WRITE
                    
            INCF    ADR, F
            MOVF    TEMP_HI,W
            CALL    E_WRITE

            INCF    ADR, F
            MOVF    TEMP_LI, W
            CALL    E_WRITE
                    
            INCF    ADR, F
            MOVF    TEMP_HI, W
            CALL    E_WRITE
            

            MOVLW   D'50'           ;SCHALTIMPULS IST GRSSER ALS 5SEC ALSO OK
            CALL    ZS100MS
            GOTO    ANFANG3

;----------------------------------------------------
MINMAX      CALL    LCD_LOE
            CALL    AUSSEN
            CALL    TEMPERAT_A
            CALL    MAX_A
;TEMPERATUR MAX-AUSSEN AUSLESEN + ANZEIGEN
            MOVLW   0X00
            MOVWF   ADR
            CALL    E_READ
            MOVWF   TEMP_L

            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_H

            BCF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE

            MOVLW   D'40'
            CALL    ZS100MS
;ENDE-----------

            CALL    LCD_LOE
            MOVLW   D'5'
            CALL    ZS100MS

            CALL    AUSSEN
            CALL    TEMPERAT_A
            CALL    MIN_A
;TEMPERATUR MIN-AUSSEN AUSLESEN + ANZEIGEN
            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_L

            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_H

            BCF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE

            MOVLW   D'40'
            CALL    ZS100MS
;ENDE-----------

            CALL    LCD_LOE
            MOVLW   D'5'
            CALL    ZS100MS

            CALL    INNEN
            CALL    TEMPERAT_I
            CALL    MAX_I
;TEMPERATUR MAX-INNEN AUSLESEN + ANZEIGEN
            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_L

            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_H

            BSF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE

            MOVLW   D'40'
            CALL    ZS100MS
;ENDE-----------

            CALL    LCD_LOE
            MOVLW   D'5'
            CALL    ZS100MS

            CALL    INNEN
            CALL    TEMPERAT_I
            CALL    MIN_I
;TEMPERATUR MIN-INNEN AUSLESEN + ANZEIGEN
            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_L

            INCF    ADR, F
            MOVF    ADR, W
            CALL    E_READ
            MOVWF   TEMP_H

            BSF     IA, BIT0
            CALL    BCD_ANZEIGE             ;HEX ZU BCD UMRECHNEN UND ANZEIGE

            MOVLW   D'40'
            CALL    ZS100MS
;ENDE-----------

            GOTO    ANFANG3



;*****************************************************************************
;*                        UNTERPROGRAMM MIN+MAX VERGLEICH                    *
;*                             IN EEPROM-SPEICHER ERGEBNIS ABSPEICHERN       *
;*                              30.09.11                                     *
;*****************************************************************************
VERGLEICH
;MAX AUSSEN
                MOVLW   0X00                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                CALL    E_READ
                MOVWF   TEMP_LSP            ;LSP AUSLESEN
                MOVLW   0X01                ;EEPROM-ADRESSE SETZEN
                CALL    E_READ
                MOVWF   TEMP_HSP            ;HSP AUSLESEN VORZEICHEN

;VORZEICHEN VERGLEICH
                MOVF    TEMP_HSP, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSS   STATUS, Z
                GOTO    MAX2                ;HSP IST 0X00 PLUS TEMP.
                MOVF    TEMP_HA, W          ;HSP IST 0X80 MINUS TEMP.
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    MAX1                ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                GOTO    MAXSP               ;HSP IST 0X80 (MINUS-TEMP) + HA IST 0X00 (PLUS-TEMP) -->GLEICH SPEICHERN 

MAX2            MOVF    TEMP_HA, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    MIN0                ;HSP IST 0X00 (PLUS-TEMP) + HA IST 0X80 (MINUS-TEMP) -->GLEICH NICHT SPEICHERN ZU MIN AUSSEN
                GOTO    MAX4                ;HSP + HA IST 0X00 (PLUS-TEMP) ZU MAX4

;;;;;;;;;;;;;;;;;;;;;;
MAX1            MOVF    TEMP_HA, W          ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG ZUM ENDE --ES GEHT WEITER MIT MAXHM
                GOTO    MAXHM

                MOVF    TEMP_HA, W          
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA IST CARRY HIGH   -->ES GEHT WEITER MIT MAXHM
                GOTO    MIN0 ; MAXHM
                GOTO    MAXSP               ;BEI LSP <(KLEINER) LA IST CARRY LOW   -->ES GEHT WEITER MIT SPEICHERN

MAX4            MOVF    TEMP_HA, W          ;HSP + HA IST 0X00 (PLUS-TEMP)
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG ZUM ENDE --ES GEHT WEITER MIT MIN AUSSEN
                GOTO    MAXHP

                MOVF    TEMP_HA, W
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA  IST CARRY HIGH   -->
                GOTO    MIN0
                GOTO    MAXSP               ;BEI HSP < (KLEINER) HA WAR CARRY LOW -->ES GEHT WEITER MIT SPEICHERN 

;;;;;;;;;;;;;;;;;;;
MAXHM           MOVF    TEMP_LA, W          ;BEI MINUS          
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->ES GEHT WEITER MIT MIN AUSSEN
                GOTO    MIN0

                MOVF    TEMP_LA, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->SPEICHERN
                GOTO    MIN0
                GOTO    MAXSP               ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->ES GEHT WEITER MIT MIN AUSSEN

MAXHP           MOVF    TEMP_LA, W          ;BEI PLUS            
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->ES GEHT WEITER MIT MIN AUSSEN
                GOTO    MIN0

                MOVF    TEMP_LA, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->ES GEHT WEITER MIT MIN AUSSEN  
                GOTO    MIN0 
;                GOTO    MAXSP                ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->ES GEHT WEITER MIT SPEICHERN
;;;;;;;;;;;;;;;;;;;


MAXSP           MOVLW   0X00                ;HIER EEPROM-ADRESSE AN DEN ANFANG SETZEN
                MOVWF   ADR
                MOVF    TEMP_LA, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

                INCF    ADR, F
                MOVF    TEMP_HA, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;MIN AUSSEN 
MIN0            MOVLW   0X02                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                CALL    E_READ
                MOVWF   TEMP_LSP            ;LSP AUSLESEN
                MOVLW   0X03                ;EEPROM-ADRESSE SETZEN
                CALL    E_READ
                MOVWF   TEMP_HSP            ;HSP AUSLESEN VORZEICHEN

;VORZEICHEN VERGLEICH
                MOVF    TEMP_HSP, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSS   STATUS, Z
                GOTO    MIN2                ;HSP IST 0X00 PLUS TEMP.
                MOVF    TEMP_HA, W          ;HSP IST 0X80 MINUS TEMP.
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    MIN1                ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                GOTO    IMAX0               ;HSP IST 0X80 (MINUS-TEMP) + HA IST 0X00 (PLUS-TEMP) -->NICHT GLEICH SPEICHERN 

MIN2            MOVF    TEMP_HA, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    MINSP               ;HSP IST 0X00 (PLUS-TEMP) + HA IST 0X80 (MINUS-TEMP) -->GLEICH SPEICHERN 
                GOTO    MIN4                ;HSP + HA IST 0X00 (PLUS-TEMP) ZU MAX4

;;;;;;;;;;;;;;;;;;;;;;
MIN1            MOVF    TEMP_HA, W          ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA 
                GOTO    MINHM

                MOVF    TEMP_HA, W          
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA IST CARRY HIGH   -->
                GOTO    MINSP
                GOTO    IMAX0 ;MINHM               ;BEI HSP <(KLEINER) HA IST CARRY LOW   -->

MIN4            MOVF    TEMP_HA, W          ;HSP + HA IST 0X00 (PLUS-TEMP)
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG 
                GOTO    MINHP

                MOVF    TEMP_HA, W
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA  IST CARRY HIGH   -->
                GOTO    MINSP  
                GOTO    IMAX0               ;BEI HSP < (KLEINER) HA WAR CARRY LOW -->

;;;;;;;;;;;;;;;;;;;
MINHM           MOVF    TEMP_LA, W          ;BEI MINUS          
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->
                GOTO    IMAX0

                MOVF    TEMP_LA, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->
                GOTO    MINSP
                GOTO    IMAX0               ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->

MINHP           MOVF    TEMP_LA, W          ;BEI PLUS            
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->
                GOTO    IMAX0

                MOVF    TEMP_LA, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->  
                GOTO    MINSP 
                GOTO    IMAX0               ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->
;;;;;;;;;;;;;;;;;;;

MINSP           MOVLW   0X02                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                MOVWF   ADR
                MOVF    TEMP_LA, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

                INCF    ADR, F
                MOVF    TEMP_HA, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;MAX INNEN
IMAX0           MOVLW   0X04                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                CALL    E_READ
                MOVWF   TEMP_LSP            ;LSP AUSLESEN
                MOVLW   0X05                ;EEPROM-ADRESSE SETZEN
                CALL    E_READ
                MOVWF   TEMP_HSP            ;HSP AUSLESEN VORZEICHEN

;VORZEICHEN VERGLEICH
;VORZEICHEN VERGLEICH
                MOVF    TEMP_HSP, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSS   STATUS, Z
                GOTO    IMAX2                ;HSP IST 0X00 PLUS TEMP.
                MOVF    TEMP_HI, W          ;HSP IST 0X80 MINUS TEMP.
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    IMAX1                ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                GOTO    IMAXSP               ;HSP IST 0X80 (MINUS-TEMP) + HA IST 0X00 (PLUS-TEMP) -->GLEICH SPEICHERN 

IMAX2            MOVF    TEMP_HI, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    IMIN0                ;HSP IST 0X00 (PLUS-TEMP) + HA IST 0X80 (MINUS-TEMP) -->GLEICH NICHT SPEICHERN ZU MIN AUSSEN
                GOTO    IMAX4                ;HSP + HA IST 0X00 (PLUS-TEMP) ZU MAX4

;;;;;;;;;;;;;;;;;;;;;;
IMAX1            MOVF    TEMP_HI, W          ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG ZUM ENDE --ES GEHT WEITER MIT MAXHM
                GOTO    IMAXHM

                MOVF    TEMP_HI, W          
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA IST CARRY HIGH   -->ES GEHT WEITER MIT MAXHM
                GOTO    IMIN0   ;IMAXHM
                GOTO    IMAXSP               ;BEI LSP <(KLEINER) LA IST CARRY LOW   -->ES GEHT WEITER MIT SPEICHERN

IMAX4            MOVF    TEMP_HI, W          ;HSP + HA IST 0X00 (PLUS-TEMP)
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG ZUM ENDE --ES GEHT WEITER MIT MIN AUSSEN
                GOTO    IMAXHP

                MOVF    TEMP_HI, W
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA  IST CARRY HIGH   -->
                GOTO    IMIN0
                GOTO    IMAXSP               ;BEI HSP < (KLEINER) HA WAR CARRY LOW -->ES GEHT WEITER MIT SPEICHERN 

;;;;;;;;;;;;;;;;;;;
IMAXHM           MOVF    TEMP_LI, W          ;BEI MINUS          
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->ES GEHT WEITER MIT MIN AUSSEN
                GOTO    IMIN0

                MOVF    TEMP_LI, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->SPEICHERN
                GOTO    IMIN0 
                GOTO    IMAXSP              ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->ES GEHT WEITER MIT MIN AUSSEN

IMAXHP           MOVF    TEMP_LI, W          ;BEI PLUS            
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->ES GEHT WEITER MIT MIN AUSSEN
                GOTO    IMIN0

                MOVF    TEMP_LI, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->ES GEHT WEITER MIT MIN AUSSEN  
                GOTO    IMIN0 
;                GOTO    MAXSP                ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->ES GEHT WEITER MIT SPEICHERN
;;;;;;;;;;;;;;;;;;;


IMAXSP          MOVLW   0X04                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                MOVWF   ADR
                MOVF    TEMP_LI, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

                INCF    ADR, F
                MOVF    TEMP_HI, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;MIN INNEN 
IMIN0           MOVLW   0X06                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                CALL    E_READ
                MOVWF   TEMP_LSP            ;LSP AUSLESEN
                MOVLW   0X07                ;EEPROM-ADRESSE SETZEN
                CALL    E_READ
                MOVWF   TEMP_HSP            ;HSP AUSLESEN VORZEICHEN

;VORZEICHEN VERGLEICH
;VORZEICHEN VERGLEICH
                MOVF    TEMP_HSP, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSS   STATUS, Z
                GOTO    IMIN2                ;HSP IST 0X00 PLUS TEMP.
                MOVF    TEMP_HI, W          ;HSP IST 0X80 MINUS TEMP.
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    IMIN1                ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                GOTO    IIMAX0               ;HSP IST 0X80 (MINUS-TEMP) + HA IST 0X00 (PLUS-TEMP) -->NICHT GLEICH SPEICHERN 

IMIN2            MOVF    TEMP_HI, W
                ANDLW   0X80
                SUBLW   0X80
                BTFSC   STATUS, Z
                GOTO    IMINSP               ;HSP IST 0X00 (PLUS-TEMP) + HA IST 0X80 (MINUS-TEMP) -->GLEICH SPEICHERN 
                GOTO    IMIN4                ;HSP + HA IST 0X00 (PLUS-TEMP) ZU MAX4

;;;;;;;;;;;;;;;;;;;;;;
IMIN1            MOVF    TEMP_HI, W          ;HA + HSP MIT 0X80 BEIDE MINUS-TEMP
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA 
                GOTO    IMINHM

                MOVF    TEMP_HI, W          
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA IST CARRY HIGH   -->
                GOTO    IMINSP
                GOTO    IIMAX0 ;IMINHM               ;BEI HSP <(KLEINER) HA IST CARRY LOW   -->

IMIN4            MOVF    TEMP_HI, W          ;HSP + HA IST 0X00 (PLUS-TEMP)
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, Z           ;BEI HSP == HA SPRUNG 
                GOTO    IMINHP

                MOVF    TEMP_HI, W
                SUBWF   TEMP_HSP, W
                BTFSC   STATUS, C           ;BEI HSP >=(GRER GLEICH) HA  IST CARRY HIGH   -->
                GOTO    IMINSP
                GOTO    IIMAX0              ;BEI HSP < (KLEINER) HA WAR CARRY LOW -->

;;;;;;;;;;;;;;;;;;;
IMINHM           MOVF    TEMP_LI, W          ;BEI MINUS          
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->
                GOTO    IIMAX0

                MOVF    TEMP_LI, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->
                GOTO    IMINSP
                GOTO    IIMAX0               ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->

IMINHP           MOVF    TEMP_LI, W          ;BEI PLUS            
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, Z           ;BEI LSP == LA -->
                GOTO    IIMAX0

                MOVF    TEMP_LI, W
                SUBWF   TEMP_LSP, W
                BTFSC   STATUS, C           ;BEI LSP >=(GRER GLEICH) LA  IST CARRY HIGH   -->  
                GOTO    IMINSP 
                GOTO    IIMAX0               ;BEI LSP < (KLEINER) LA WAR CARRY LOW -->
;;;;;;;;;;;;;;;;;;;

IMINSP          MOVLW   0X06                ;EEPROM-ADRESSE AN DEN ANFANG SETZEN
                MOVWF   ADR
                MOVF    TEMP_LI, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

                INCF    ADR, F
                MOVF    TEMP_HI, W          ;DERZEIT AKTUELLER GEMESSENEN WERT ABSPEICHERN
                CALL    E_WRITE

IIMAX0          RETURN


;*****************************************************************************
;*                        UNTERPROGRAMM EMPFANGEN RX VON EINEM BYTE          *
;*                             ERGEBNIS STEHT AM SCHLUSS IN W FR AUSSEN     *
;*                              03.02.11                                     *
;*****************************************************************************
RXA             MOVLW   D'8'                     ;8 BITS SOLLEN EMPFANGEN WERDEN  
                MOVWF   ZAE
 
                CLRF    HELP1

RXANFA          TXRX_OUTPUTA                    ;0SEC
;ZEITSCHLEIFE 4SEC
                MOVLW   D'3'                    ;T=4
                MOVWF   HELP2                   ;T=4
L511A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L511A                   ;T=8 
;GES.: (16T*3) * 0,0625SEC=3,0SEC + (16T*0,0625SEC)=3,0SEC+1,0SEC=4SEC
;ENDE ZEITSCHLEIFE 
                TXRX_INPUTA                      

;ZEITSCHLEIFE 5SEC
                MOVLW   D'4'                    ;T=4
                MOVWF   HELP2                   ;T=4
L512A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L512A                   ;T=8 
;GES.: (16T*4) * 0,0625SEC=4SEC + (16T*0,0625SEC)=4SEC+1,0SEC=5SEC
;ENDE ZEITSCHLEIFE   

                BTFSS   PORTA, TXRXA
                GOTO    W513A
                BSF     HELP1, BIT7

W513A           DECFSZ  ZAE, F
                GOTO    W514A

;ZEITSCHLEIFE 48SEC
                MOVLW   D'47'                   ;T=4
                MOVWF   HELP2                   ;T=4
L613A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L613A                   ;T=8 
;GES.: (16T*47) * 0,0625SEC=47SEC + (16T*0,0625SEC)=47SEC+1,0SEC=48SEC
;ENDE ZEITSCHLEIFE 

                MOVF    HELP1, W
                RETURN

W514A           BCF     STATUS, C
                RRF     HELP1, F

;ZEITSCHLEIFE 48SEC
                MOVLW   D'47'                   ;T=4
                MOVWF   HELP2                   ;T=4
L513A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L513A                   ;T=8 
;GES.: (16T*47) * 0,0625SEC=47SEC + (16T*0,0625SEC)=47SEC+1,0SEC=48SEC
;ENDE ZEITSCHLEIFE 

                GOTO    RXANFA

;*****************************************************************************
;*                        UNTERPROGRAMM EMPFANGEN RX VON EINEM BYTE          *
;*                             ERGEBNIS STEHT AM SCHLUSS IN W FR INNEN      *
;*                              03.02.11                                     *
;*****************************************************************************
RXI             MOVLW   D'8'                     ;8 BITS SOLLEN EMPFANGEN WERDEN  
                MOVWF   ZAE
 
                CLRF    HELP1

RXANFI          TXRX_OUTPUTI                    ;0SEC
;ZEITSCHLEIFE 4SEC
                MOVLW   D'3'                    ;T=4
                MOVWF   HELP2                   ;T=4
L511I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L511I                   ;T=8 
;GES.: (16T*3) * 0,0625SEC=3,0SEC + (16T*0,0625SEC)=3,0SEC+1,0SEC=4SEC
;ENDE ZEITSCHLEIFE 
                TXRX_INPUTI                      

;ZEITSCHLEIFE 5SEC
                MOVLW   D'4'                    ;T=4
                MOVWF   HELP2                   ;T=4
L512I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L512I                   ;T=8 
;GES.: (16T*4) * 0,0625SEC=4SEC + (16T*0,0625SEC)=4SEC+1,0SEC=5SEC
;ENDE ZEITSCHLEIFE   

                BTFSS   PORTA, TXRXI
                GOTO    W513I
                BSF     HELP1, BIT7

W513I           DECFSZ  ZAE, F
                GOTO    W514I

;ZEITSCHLEIFE 48SEC
                MOVLW   D'47'                   ;T=4
                MOVWF   HELP2                   ;T=4
L613I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L613I                   ;T=8 
;GES.: (16T*47) * 0,0625SEC=47SEC + (16T*0,0625SEC)=47SEC+1,0SEC=48SEC
;ENDE ZEITSCHLEIFE 

                MOVF    HELP1, W
                RETURN

W514I           BCF     STATUS, C
                RRF     HELP1, F

;ZEITSCHLEIFE 48SEC
                MOVLW   D'47'                   ;T=4
                MOVWF   HELP2                   ;T=4
L513I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L513I                   ;T=8 
;GES.: (16T*47) * 0,0625SEC=47SEC + (16T*0,0625SEC)=47SEC+1,0SEC=48SEC
;ENDE ZEITSCHLEIFE 

                GOTO    RXANFI
;*****************************************************************************
;*                        UNTERPROGRAMM SENDEN TX VON EINEM BYTE             *
;*                              03.02.11       FR AUSSEN                    *
;*****************************************************************************
TXA             MOVWF   HELP1

                MOVLW   D'8'                     ;8 BITS SOLLEN BERTRAGEN WERDEN  
                MOVWF   ZAE

TXANFA          RRF     HELP1, F                 ;BIT0 RECHTS SCHIEBEN BERS CARRY-REGISTER
                BTFSC   STATUS, C                ;SEQUENZ AUSWHLEN FR LOW- UND HIGH-BIT
                GOTO    HIGHHA

;FUER LOW SENDEN
LOOWA           TXRX_OUTPUTA                    ;0SEC
;ZEITSCHLEIFE 62SEC
                MOVLW   D'61'                   ;T=4
                MOVWF   HELP2                   ;T=4
L111A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L111A                   ;T=8 
;GES.: (16T*61) * 0,0625SEC=61,0SEC + (16T*0,0625SEC)=61,0SEC+1,0SEC=62SEC
;ENDE ZEITSCHLEIFE 
                TXRX_INPUTA                      

;ZEITSCHLEIFE 10SEC
                MOVLW   D'9'                    ;T=4
                MOVWF   HELP2                   ;T=4
L112A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L112A                   ;T=8 
;GES.: (16T*9) * 0,0625SEC=9SEC + (16T*0,0625SEC)=9SEC+1,0SEC=10SEC
;ENDE ZEITSCHLEIFE   

                DECFSZ  ZAE, F                     
                GOTO    TXANFA
                RETURN

;-------------------------
;FUER HIGH SENDEN
HIGHHA          TXRX_OUTPUTA
;ZEITSCHLEIFE 6SEC
                MOVLW   D'5'                    ;T=4
                MOVWF   HELP2                   ;T=4
L211A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L211A                    ;T=8 
;GES.: (16T*5) * 0,0625SEC=5,0SEC + (16T*0,0625SEC)=5,0SEC+1,0SEC=6,0SEC
;ENDE ZEITSCHLEIFE     
                TXRX_INPUTA                      

;ZEITSCHLEIFE 66SEC
                MOVLW   D'65'                   ;T=4
                MOVWF   HELP2                   ;T=4
L212A           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L212A                   ;T=8 
;GES.: (16T*65) * 0,0625SEC=65,0SEC + (16T*0,0625SEC)=65,0SEC+1,0SEC=66,0SEC
;ENDE ZEITSCHLEIFE    

                DECFSZ  ZAE, F                     
                GOTO    TXANFA
                RETURN

;BEI DEN ZEITEN HATTE ICH SCHON VIEL PROBIERT 
;DIE SUMME IST JETZT IMMER 72SEC == BEI LOW 62SEC+10SEC=72SEC    UND    BEI HIGH 6SEC+66SEC=72SEC


;*****************************************************************************
;*                        UNTERPROGRAMM SENDEN TX VON EINEM BYTE             *
;*                              03.02.11       FR INNEN                     *
;*****************************************************************************
TXI             MOVWF   HELP1

                MOVLW   D'8'                     ;8 BITS SOLLEN BERTRAGEN WERDEN  
                MOVWF   ZAE

TXANFI          RRF     HELP1, F                 ;BIT0 RECHTS SCHIEBEN BERS CARRY-REGISTER
                BTFSC   STATUS, C                ;SEQUENZ AUSWHLEN FR LOW- UND HIGH-BIT
                GOTO    HIGHHI

;FUER LOW SENDEN
LOOWI           TXRX_OUTPUTI                    ;0SEC
;ZEITSCHLEIFE 62SEC
                MOVLW   D'61'                   ;T=4
                MOVWF   HELP2                   ;T=4
L111I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L111I                   ;T=8 
;GES.: (16T*61) * 0,0625SEC=61,0SEC + (16T*0,0625SEC)=61,0SEC+1,0SEC=62SEC
;ENDE ZEITSCHLEIFE 
                TXRX_INPUTI                      

;ZEITSCHLEIFE 10SEC
                MOVLW   D'9'                    ;T=4
                MOVWF   HELP2                   ;T=4
L112I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L112I                   ;T=8 
;GES.: (16T*9) * 0,0625SEC=9SEC + (16T*0,0625SEC)=9SEC+1,0SEC=10SEC
;ENDE ZEITSCHLEIFE   

                DECFSZ  ZAE, F                     
                GOTO    TXANFI
                RETURN

;-------------------------
;FUER HIGH SENDEN
HIGHHI          TXRX_OUTPUTI
;ZEITSCHLEIFE 6SEC
                MOVLW   D'5'                    ;T=4
                MOVWF   HELP2                   ;T=4
L211I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L211I                    ;T=8 
;GES.: (16T*5) * 0,0625SEC=5,0SEC + (16T*0,0625SEC)=5,0SEC+1,0SEC=6,0SEC
;ENDE ZEITSCHLEIFE     
                TXRX_INPUTI                      

;ZEITSCHLEIFE 66SEC
                MOVLW   D'65'                   ;T=4
                MOVWF   HELP2                   ;T=4
L212I           NOP                             ;T=4
                DECFSZ  HELP2, F                ;T=4 BEI SPRUNG T=8 
                GOTO    L212I                   ;T=8 
;GES.: (16T*65) * 0,0625SEC=65,0SEC + (16T*0,0625SEC)=65,0SEC+1,0SEC=66,0SEC
;ENDE ZEITSCHLEIFE    

                DECFSZ  ZAE, F                     
                GOTO    TXANFI
                RETURN

;BEI DEN ZEITEN HATTE ICH SCHON VIEL PROBIERT 
;DIE SUMME IST JETZT IMMER 72SEC == BEI LOW 62SEC+10SEC=72SEC    UND    BEI HIGH 6SEC+66SEC=72SEC

;*****************************************************************************
;*                        UNTERPROGRAMM RESET                                *
;*                              03.02.11     FR AUSSEN                      *
;*****************************************************************************
RESETA          TXRX_OUTPUTA
;MINDESTENS 480SEC (GENOMMEN 485SEC) FUER LOW AN RA0 SENDEN
;ZEITSCHLEIFE 485SEC
                MOVLW   D'215'                  ;T=4
                MOVWF   HELP1                   ;T=4

LO1A            NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG T=8 
                GOTO    LO1A                    ;T=8 
;GES.: (36T*215) * 0,0625SEC=483,75SEC + (16T*0,0625SEC)=483,75SEC+1,0SEC=484,75SEC
;ENDE ZEITSCHLEIFE              
                TXRX_INPUTA


;WARTEN 15-60SEC IST ES NOCH HIGH AN RA0(TXRX) , WARTEN BIS ES LOW WIRD
;WENN ES NICHT LOW WIRD,  KOMMT EIN ERNEUTER LOW-IMPULS NACH 5MSEC, ALSO SPRUNG ZUM ANFANG ZU RESET

;ZEITSCHLEIFEN-ABFRAGE 100SEC
                MOVLW   D'66'                   ;T=4
                MOVWF   HELP1                   ;T=4
L311A           BTFSS   PORTA, TXRXA            ;(T=4) BEI SPRUNG T=8
                GOTO    WARTE1A
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG (T=8) 
                GOTO    L311A                   ;T=8 
;GES.: (24T*66) * 0,0625SEC=99,0SEC + (16T*0,0625SEC)=99,0SEC+1,0SEC=100,0SEC
;ENDE ZEITSCHLEIFE    

;100SEC SIND ZU ENDE, ES KAM KEIN LOW-IMPULS, ES GEHT NACH 50MSEC ERNEUT ZUM RESET AM ANFANG
;50MSEC ZEITSCHLEIFE
                MOVLW   D'50'                   ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 50 ALSO---> 50 * 1MSEC=50MSEC
                CALL    ZS1MS                   ;ZEITSCHLEIFE ABARBEITEN
                GOTO    RESETA
                
;JETZT IST ES LOW (60 BIS 240SEC LANG) ALSO DER DS18B20SENDET EIN LOW ZURCK
;WARTEN BIS ES HIGH WIRD
;ES WIRD NICHT KONTROLLIERT OB DER LOW-IMPULS NUN WIRKLICH ZWISCHEN 60 UND 240SEC LIEGT, ALSO KEINE LAENGEN-ZEIT-MESSUNG
WARTE1A         BTFSS   PORTA, TXRXA
                GOTO    WARTE1A

;NACH MINDISTENS GESAMT 15SEC+60SEC+451SEC = HIER (526SEC)
;(SIEHE IM DATASHEET) KANN ES DANN WEITER GEHEN MIT DEM DATEN SENDEN IN DEN DS18B20

;ZEITSCHLEIFE 451SEC
                MOVLW   D'200'                  ;T=4
                MOVWF   HELP1                   ;T=4

LO2A            NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG T=8 
                GOTO    LO2A                    ;T=8 
;GES.: (36T*200) * 0,0625SEC=450,0SEC + (16T*0,0625SEC)=450SEC+1,0SEC=451SEC
;ENDE ZEITSCHLEIFE 
;HIER GILT: DIE MIN. 15SEC + MIN. 60SEC = 75SEC +  X  = MUESSEN MIN. 480SEC SEIN /// HIER IST X=451SEC ALSO GESAMT:526SEC
;MIT X=451SEC IST ALSO ALLES OK



;DIES HAT ALLES EXAKT FUNKTIONIERT
;DER DS18B20ANTWORTET MIT EINEM LOW-IMPULS VON 60-240SEC
;ES BLEIBT NICHT HNGEN

                RETURN


;*****************************************************************************
;*                        UNTERPROGRAMM RESET                                *
;*                              03.02.11       FR INNEN                     *
;*****************************************************************************
RESETI          TXRX_OUTPUTI
;MINDESTENS 480SEC (GENOMMEN 485SEC) FUER LOW AN RA0 SENDEN
;ZEITSCHLEIFE 485SEC
                MOVLW   D'215'                  ;T=4
                MOVWF   HELP1                   ;T=4

LO1I            NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG T=8 
                GOTO    LO1I                    ;T=8 
;GES.: (36T*215) * 0,0625SEC=483,75SEC + (16T*0,0625SEC)=483,75SEC+1,0SEC=484,75SEC
;ENDE ZEITSCHLEIFE              
                TXRX_INPUTI


;WARTEN 15-60SEC IST ES NOCH HIGH AN RA0(TXRX) , WARTEN BIS ES LOW WIRD
;WENN ES NICHT LOW WIRD,  KOMMT EIN ERNEUTER LOW-IMPULS NACH 5MSEC, ALSO SPRUNG ZUM ANFANG ZU RESET

;ZEITSCHLEIFEN-ABFRAGE 100SEC
                MOVLW   D'66'                   ;T=4
                MOVWF   HELP1                   ;T=4
L311I           BTFSS   PORTA, TXRXI             ;(T=4) BEI SPRUNG T=8
                GOTO    WARTE1I
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG (T=8) 
                GOTO    L311I                   ;T=8 
;GES.: (24T*66) * 0,0625SEC=99,0SEC + (16T*0,0625SEC)=99,0SEC+1,0SEC=100,0SEC
;ENDE ZEITSCHLEIFE    

;100SEC SIND ZU ENDE, ES KAM KEIN LOW-IMPULS, ES GEHT NACH 50MSEC ERNEUT ZUM RESET AM ANFANG
;50MSEC ZEITSCHLEIFE
                MOVLW   D'50'                   ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 50 ALSO---> 50 * 1MSEC=50MSEC
                CALL    ZS1MS                   ;ZEITSCHLEIFE ABARBEITEN
                GOTO    RESETI
                
;JETZT IST ES LOW (60 BIS 240SEC LANG) ALSO DER DS18B20SENDET EIN LOW ZURCK
;WARTEN BIS ES HIGH WIRD
;ES WIRD NICHT KONTROLLIERT OB DER LOW-IMPULS NUN WIRKLICH ZWISCHEN 60 UND 240SEC LIEGT, ALSO KEINE LAENGEN-ZEIT-MESSUNG
WARTE1I         BTFSS   PORTA, TXRXI
                GOTO    WARTE1I

;NACH MINDISTENS GESAMT 15SEC+60SEC+451SEC = HIER (526SEC)
;(SIEHE IM DATASHEET) KANN ES DANN WEITER GEHEN MIT DEM DATEN SENDEN IN DEN DS18B20

;ZEITSCHLEIFE 451SEC
                MOVLW   D'200'                  ;T=4
                MOVWF   HELP1                   ;T=4

LO2I            NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                NOP                             ;T=4
                DECFSZ  HELP1, F                ;T=4 BEI SPRUNG T=8 
                GOTO    LO2I                    ;T=8 
;GES.: (36T*200) * 0,0625SEC=450,0SEC + (16T*0,0625SEC)=450SEC+1,0SEC=451SEC
;ENDE ZEITSCHLEIFE 
;HIER GILT: DIE MIN. 15SEC + MIN. 60SEC = 75SEC +  X  = MUESSEN MIN. 480SEC SEIN /// HIER IST X=451SEC ALSO GESAMT:526SEC
;MIT X=451SEC IST ALSO ALLES OK



;DIES HAT ALLES EXAKT FUNKTIONIERT
;DER DS18B20ANTWORTET MIT EINEM LOW-IMPULS VON 60-240SEC
;ES BLEIBT NICHT HNGEN

                RETURN

;*****************************************************************************
;*         UNTERPROGRAMM MIT TIMER0 ALS ZEITSCHLEIFE FUER  1MSEC ....        *
;*                              23.08.10                                     *
;*****************************************************************************
ZS1MS                   MOVWF   HELP1                   ;DER INHALT VON W WIRD IN "HELP1" GELADEN

T01                     MOVLW   D'240'
                        MOVWF   TMR0                    ;DER FESTE RAM-SPEICHER (TMR0) WIRD MIT 240 GELADEN
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                
T02                     BTFSS   INTCON, T0IF            ;WENN DIE ZEIT VON 10MSEC ABGELAUFEN IST, WIRD T0IF HIGH (1) UND 
                        GOTO    T02                     ;ES ERFOLGT EIN SPRUNG ZU "DECFSC MERK, F"
                        
                        DECFSZ  HELP1, F                ;ES WIRD DER WERT IN "HELP1" ABWRTS GEZHLT, BIS ER (0) IST UND
                        GOTO    T01                     ;DANN ERFOLGT EIN SPRUNG ZU "RETURN"-ENDE DES UNTERPROGRAMMS
                        RETURN

;ZEITBERECHNUNG FUER 1MS
;(HIER MUSS MAN IN DEM DATENBLATT (DATASHEET) BEI WWW.MICROCHIP.COM (EIN *.PDF-FILE) NACHSEHEN, WIE ALLES FUNKTIONIERT)

;PRESCALER 256X  *  SYSTEM 4X  * TAKTZEIT ====> 256*4*0.0625S=64S    /DIE TAKTZEIT ERGIBT SICH AUS DEM INTERNEN OSZILLATOR
;1MS : 64S= 15,6                                                      /T = 1 : F  (F)-OSZILLATORFREQUENZ
;ALSO
;256S*D'16'=CA.1MS
;TMR0 ALSO D'256'-D'16'=D'240' ENTSPRICHT CA. 1MS
;240 DEZIMAL OBEN EINTRAGEN

;*****************************************************************************
;*         UNTERPROGRAMM MIT TIMER0 ALS ZEITSCHLEIFE FUER  100MSEC ....      *
;*                              23.08.10                                     *
;*****************************************************************************
ZS100MS                 MOVWF   HELP1                   ;DER INHALT VON W WIRD IN "HELP1" GELADEN

T100                    MOVLW   D'100'                  ;100MAL WIRD DIE 1MSEC ZEIT ABGEARBEITET, HIER WIRD GELADEN
                        MOVWF   HELP2                   ;ALSO 100MSEC

T101                    MOVLW   D'240'
                        MOVWF   TMR0                    ;DER FESTE RAM-SPEICHER (TMR0) WIRD MIT 240 GELADEN
                        BCF     INTCON, T0IF            ;OVERFLOW = 0
                
T102                    BTFSS   INTCON, T0IF            ;WENN DIE ZEIT VON 10MSEC ABGELAUFEN IST, WIRD T0IF HIGH (1) UND 
                        GOTO    T102                    ;ES ERFOLGT EIN SPRUNG ZU "DECFSC MERK, F"
                        
                        DECFSZ  HELP2, F
                        GOTO    T101

                        DECFSZ  HELP1, F                ;ES WIRD DER WERT IN "HELP1" ABWRTS GEZHLT, BIS ER (0) IST UND
                        GOTO    T100                    ;DANN ERFOLGT EIN SPRUNG ZU "RETURN"-ENDE DES UNTERPROGRAMMS

                        RETURN

;*****************************************************************************
;*                              GEHT GLEICH LOS AUF DOT-LCD-MODUL            *
;*                              27.09.11                                     *
;*****************************************************************************
GEHT        MOVLW   H'80'                   ;ADRESSE 0 
            CALL    BEFEHL
            MOVLW   H'47'
            CALL    DATEN

            MOVLW   H'81'                   ;ADRESSE 1
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'82'                   ;ADRESSE 2
            CALL    BEFEHL
            MOVLW   H'68'
            CALL    DATEN

            MOVLW   H'83'                   ;ADRESSE 3
            CALL    BEFEHL
            MOVLW   H'74'
            CALL    DATEN

            MOVLW   H'84'                   ;ADRESSE 4
            CALL    BEFEHL
            MOVLW   H'20'
            CALL    DATEN

            MOVLW   H'85'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'67'
            CALL    DATEN

            MOVLW   H'86'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'6C'
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'88'                   ;ADRESSE 8
            CALL    BEFEHL
            MOVLW   H'69'
            CALL    DATEN

            MOVLW   H'89'                   ;ADRESSE 9
            CALL    BEFEHL
            MOVLW   H'63'
            CALL    DATEN

            MOVLW   H'8A'                   ;ADRESSE A
            CALL    BEFEHL
            MOVLW   H'68'
            CALL    DATEN

            MOVLW   H'8B'                   ;ADRESSE B
            CALL    BEFEHL
            MOVLW   H'20'
            CALL    DATEN

            MOVLW   H'8C'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'6C'
            CALL    DATEN

            MOVLW   H'8D'                   ;ADRESSE D
            CALL    BEFEHL
            MOVLW   H'6F'
            CALL    DATEN

            MOVLW   H'8E'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'73'
            CALL    DATEN

;            MOVLW   H'8F'                   ;ADRESSE F
;            CALL    BEFEHL
;            MOVLW   H'73'
;            CALL    DATEN

;--------------------------------------------------------------
            MOVLW   H'C0'                   ;ADRESSE 0 
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C1'                   ;ADRESSE 1
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C2'                   ;ADRESSE 2
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C3'                   ;ADRESSE 3
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C4'                   ;ADRESSE 4
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C5'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C6'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C7'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C8'                   ;ADRESSE 8
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'C9'                   ;ADRESSE 9
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'CA'                   ;ADRESSE A
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'CB'                   ;ADRESSE B
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'CC'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'CD'                   ;ADRESSE D
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

            MOVLW   H'CE'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'2D'
            CALL    DATEN

;            MOVLW   H'CF'                   ;ADRESSE F
;            CALL    BEFEHL
;            MOVLW   H'2D'
;            CALL    DATEN

            RETURN
;*****************************************************************************
;*                              TEMPERATUR FR INNEN MAX MIN                 *
;*                              27.09.11                                     *
;*****************************************************************************
TEMPERAT_I  MOVLW   H'86'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'54'
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'88'                   ;ADRESSE 8
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'89'                   ;ADRESSE 9
            CALL    BEFEHL
            MOVLW   H'70'
            CALL    DATEN

            MOVLW   H'8A'                   ;ADRESSE A
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'8B'                   ;ADRESSE B
            CALL    BEFEHL
            MOVLW   H'72'
            CALL    DATEN

            MOVLW   H'8C'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'61'
            CALL    DATEN

            MOVLW   H'8D'                   ;ADRESSE D
            CALL    BEFEHL
            MOVLW   H'74'
            CALL    DATEN

            MOVLW   H'8E'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'75'
            CALL    DATEN

            MOVLW   H'8F'                   ;ADRESSE F
            CALL    BEFEHL
            MOVLW   H'72'
            CALL    DATEN
            RETURN

TEMPERAT_A  MOVLW   H'C6'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'54'
            CALL    DATEN

            MOVLW   H'C7'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'C8'                   ;ADRESSE 8
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'C9'                   ;ADRESSE 9
            CALL    BEFEHL
            MOVLW   H'70'
            CALL    DATEN

            MOVLW   H'CA'                   ;ADRESSE A
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'CB'                   ;ADRESSE B
            CALL    BEFEHL
            MOVLW   H'72'
            CALL    DATEN

            MOVLW   H'CC'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'61'
            CALL    DATEN

            MOVLW   H'CD'                   ;ADRESSE D
            CALL    BEFEHL
            MOVLW   H'74'
            CALL    DATEN

            MOVLW   H'CE'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'75'
            CALL    DATEN

            MOVLW   H'CF'                   ;ADRESSE F
            CALL    BEFEHL
            MOVLW   H'72'
            CALL    DATEN
            RETURN

MAX_I       MOVLW   H'80'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'81'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'61'
            CALL    DATEN

            MOVLW   H'82'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'78'
            CALL    DATEN

            MOVLW   H'83'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN
            RETURN

MIN_I       MOVLW   H'80'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'81'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'69'
            CALL    DATEN

            MOVLW   H'82'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

            MOVLW   H'83'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN
            RETURN

MAX_A       MOVLW   H'C0'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'C1'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'61'
            CALL    DATEN

            MOVLW   H'C2'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'78'
            CALL    DATEN

            MOVLW   H'C3'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN
            RETURN

MIN_A       MOVLW   H'C0'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6D'
            CALL    DATEN

            MOVLW   H'C1'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'69'
            CALL    DATEN

            MOVLW   H'C2'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

            MOVLW   H'C3'                   ;ADRESSE 
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN
            RETURN

;*****************************************************************************
;*                              LOESCHEN INNEN AUSSEN LCD-MODUL              *
;*                              27.09.11                                     *
;*****************************************************************************
RESETANZ    MOVLW   H'86'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'52'
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'45'
            CALL    DATEN

 
            MOVLW   H'88'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'53'
            CALL    DATEN

            MOVLW   H'89'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'45'
            CALL    DATEN

            MOVLW   H'8A'                   ;ADRESSE F
            CALL    BEFEHL
            MOVLW   H'54'
            CALL    DATEN
            RETURN


;*****************************************************************************
;*                              LOESCHEN INNEN AUSSEN LCD-MODUL              *
;*                              27.09.11                                     *
;*****************************************************************************
LCD_LOE     MOVLW   H'01'                   ;CLEAR DISPLAY
            CALL    BEFEHL

;3MSEC ZEITSCHLEIFE
            MOVLW   D'3'                  ;ZEITSCHLEIFE LADEN IN W MIT DEZIMAL 3 ALSO---> 3*1MSEC=3MSEC
            CALL    ZS1MS                 ;ZEITSCHLEIFE ABARBEITEN
            RETURN

AUSSEN      MOVLW   H'82'                   ;ADRESSE 2
            CALL    BEFEHL
            MOVLW   H'41'
            CALL    DATEN

            MOVLW   H'83'                   ;ADRESSE 3
            CALL    BEFEHL
            MOVLW   H'75'
            CALL    DATEN

            MOVLW   H'84'                   ;ADRESSE 4
            CALL    BEFEHL
            MOVLW   H'73'
            CALL    DATEN

            MOVLW   H'85'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'73'
            CALL    DATEN

            MOVLW   H'86'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'87'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

 
            MOVLW   H'8C'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN

            MOVLW   H'8E'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'DF'
            CALL    DATEN

            MOVLW   H'8F'                   ;ADRESSE F
            CALL    BEFEHL
            MOVLW   H'43'
            CALL    DATEN
            RETURN
;--------------------------------------------------------------
INNEN       MOVLW   H'C3'                   ;ADRESSE 3
            CALL    BEFEHL
            MOVLW   H'49'
            CALL    DATEN

            MOVLW   H'C4'                   ;ADRESSE 4
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

            MOVLW   H'C5'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

            MOVLW   H'C6'                   ;ADRESSE 6
            CALL    BEFEHL
            MOVLW   H'65'
            CALL    DATEN

            MOVLW   H'C7'                   ;ADRESSE 7
            CALL    BEFEHL
            MOVLW   H'6E'
            CALL    DATEN

 
            MOVLW   H'CC'                   ;ADRESSE C
            CALL    BEFEHL
            MOVLW   H'2E'
            CALL    DATEN

            MOVLW   H'CE'                   ;ADRESSE E
            CALL    BEFEHL
            MOVLW   H'DF'
            CALL    DATEN

            MOVLW   H'CF'                   ;ADRESSE F
            CALL    BEFEHL
            MOVLW   H'43'
            CALL    DATEN

            RETURN



;*****************************************************************************
;*                              DATEN AUSGEBEN AUF DOT-LCD-MODUL             *
;*                              04.05.10                                     *
;*****************************************************************************
DATEN       BSF     PORTA, RS               ;ALS DATEN

            MOVWF   BIT

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB


            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'4'                    ;UNGEFAEHR 4SEC ZEITSCHLEIFE
            MOVWF   HELP1
D1          NOP                             ;T=4
            DECFSZ  HELP1, F                ;T=4
            GOTO    D1                      ;T=8    GES.:16TAKTE * 0,0625SEC= 1SEC    *4 DURCHLAEUFE= 4SEC
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;---------------------------------

;LOW 4BIT AUSGEBEN
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB


            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'4'                    
            MOVWF   HELP1
DDD1        NOP                             
            DECFSZ  HELP1, F                
            GOTO    DDD1                    
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;ZEITSCHLEIFE 40SEC
            MOVLW   D'40'                   ;UNGEFAEHR 40SEC ZEITSCHLEIFE
            MOVWF   HELP1
DDDD1       NOP
            DECFSZ  HELP1, F                ;WIE OBEN  
            GOTO    DDDD1
;ENDE ZEITSCHLEIFE



            RETURN

;*****************************************************************************
;*                              BEFEHL AUSGEBEN AUF DOT-LCD-MODUL            *
;*                              04.05.10                                     *
;*****************************************************************************
BEFEHL      BCF     PORTA, RS                ;ALS BEFEHL

            MOVWF   BIT

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB

            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'4'                    ;UNGEFAEHR 8SEC ZEITSCHLEIFE
            MOVWF   HELP1
B1          NOP                             ;T=4
            DECFSZ  HELP1, F                ;T=4
            GOTO    B1                      ;T=8        GES.:16TAKTE * 0,0625SEC= 1SEC    *4 DURCHLAEUFE= 4SEC
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;---------------------------------

;LOW 4BIT AUSGEBEN
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F
            RLF     BIT, F

            MOVF    PORTB, W
            ANDLW   0X0F
            MOVWF   MERKE

            MOVF    BIT, W
            ANDLW   0XF0
            ADDWF   MERKE, W
            MOVWF   PORTB

            BSF     PORTA, EN

;ZEITSCHLEIFE 4SEC
            MOVLW   D'4'                    
            MOVWF   HELP1
BBB1        NOP                             
            DECFSZ  HELP1, F                
            GOTO    BBB1                    
;ENDE ZEITSCHLEIFE
 
            BCF     PORTA, EN

;ZEITSCHLEIFE 40SEC
            MOVLW   D'40'                   ;UNGEFAEHR 40SEC ZEITSCHLEIFE
            MOVWF   HELP1
BBBB1       NOP
            DECFSZ  HELP1, F                ;WIE OBEN  
            GOTO    BBBB1
;ENDE ZEITSCHLEIFE
 
            RETURN

;*****************************************************************************
;*                        UNTERPROGRAMM BCD-UMRECHNUNG UND ANZEIGE           *
;*                        DIES IST NUR MAL GANZ SCHNELL ZUSAMMEN GEPFUSCHT   *
;*                        DESHALB STEHEN NUR GANZ WENIGE KOMMENTARE DRIN     *
;*                              31.03.11                                     *
;*****************************************************************************
BCD_ANZEIGE CLRF    MERK


            MOVF    TEMP_L, W
            ANDLW   0X0F
            MOVWF   TEM

            MOVLW   0X00                    ;BEI X,0C GLEICH AUSGEBEN
            SUBWF   TEM, W          
            BTFSS   STATUS, Z
            GOTO    W11W

            BTFSS   IA, BIT0
            GOTO    WAB2

            MOVLW   H'CD'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'30'                   ;0.0C DIE 0
            CALL    DATEN
            BSF     MERK, BIT0
            GOTO    W00W

WAB2        MOVLW   H'8D'                   ;ADRESSE 5
            CALL    BEFEHL
            MOVLW   H'30'                   ;0.0C DIE 0
            CALL    DATEN
            BSF     MERK, BIT0
            GOTO    W00W


;BEI NICHT X,0C
W11W        BTFSS   TEMP_H, BIT7            ;BIT7 GEWHLT, ES MUSS 0X HEX SEIN BEI POSITIVER TEMPERATUR
            GOTO    WTW                    
            BSF     MERK, BIT7              ;GESETZT BEI NEGATIVEN TEMPERATUREN

WTW         BTFSS   IA, BIT0
            GOTO    WA1

            MOVLW   H'CD'                   ;ADRESSE 5
            CALL    BEFEHL  
            CALL    POLAT                   
            MOVWF   KO_ST
            MOVLW   H'04'                   ;ADRESSE SPEICHERN NEUES PCLATH
            MOVWF   PCLATH
            MOVF    KO_ST, W
            CALL    DATEN
            GOTO    W00W

WA1         MOVLW   H'8D'                   ;ADRESSE 5
            CALL    BEFEHL  
            CALL    POLAT                   
            MOVWF   KO_ST
            MOVLW   H'04'                   ;ADRESSE SPEICHERN NEUES PCLATH
            MOVWF   PCLATH
            MOVF    KO_ST, W
            CALL    DATEN

W00W        BTFSS   TEMP_H, BIT7            ;BIT7 GEWHLT, ES MUSS 0X HEX SEIN BEI POSITIVER TEMPERATUR
            GOTO    WMW1                    
            MOVF    TEMP_L, W
            MOVWF   TEM
            MOVF    TEMP_H, W
            MOVWF   TEMH
            BSF     STATUS, C               ;4MAL RECHTS
            RRF     TEMH, F
            RRF     TEM, F
            BSF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F
            BSF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F
            BSF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F

            COMF    TEM, F
            BTFSS   MERK, BIT0
            GOTO    WMW
            INCF    TEM, F
            GOTO    WMW

WMW1        MOVF    TEMP_L, W               ;POSITIVE C
            MOVWF   TEM
            MOVF    TEMP_H, W
            MOVWF   TEMH
            BCF     STATUS, C               ;4MAL RECHTS
            RRF     TEMH, F
            RRF     TEM, F
            BCF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F
            BCF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F
            BCF     STATUS, C
            RRF     TEMH, F
            RRF     TEM, F

;----------------------------
;JETZT BCD-UMWANDLUNG
WMW         CLRF    ANZ_L
            CLRF    ANZ_H

            BTFSS   TEM, BIT0
            GOTO    W001
            MOVLW   0X01
            MOVWF   ANZ_L

W001        BTFSS   TEM, BIT1
            GOTO    W002
            MOVF    ANZ_L, W
            ADDLW   0X02
            MOVWF   ANZ_L

W002        BTFSS   TEM, BIT2
            GOTO    W003
            MOVF    ANZ_L, W
            ADDLW   0X04
            MOVWF   ANZ_L

;-------

W003        BTFSS   TEM, BIT3
            GOTO    W004
            MOVLW   D'8'
            MOVWF   ZAE

W003B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W003A
            MOVF    ANZ_L, W
            ANDLW   0XF0
            ADDLW   0X10
            MOVWF   ANZ_L

W003A       DECFSZ  ZAE, F
            GOTO    W003B

;--------

W004        BTFSS   TEM, BIT4
            GOTO    W005
            MOVLW   D'16'
            MOVWF   ZAE

W004B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W004A
            MOVF    ANZ_L, W
            ANDLW   0XF0
            ADDLW   0X10
            MOVWF   ANZ_L

W004A       DECFSZ  ZAE, F
            GOTO    W004B

;--------

W005        BTFSS   TEM, BIT5
            GOTO    W006
            MOVLW   D'32'
            MOVWF   ZAE

W005B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W005A
            MOVF    ANZ_L, W
            ANDLW   0XF0
            ADDLW   0X10
            MOVWF   ANZ_L 

W005A       DECFSZ  ZAE, F
            GOTO    W005B

;--------

W006        BTFSS   TEM, BIT6
            GOTO    W007
            MOVLW   D'64'
            MOVWF   ZAE

W006B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W006A
            MOVF    ANZ_L, W
            ANDLW   0XF0
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER 
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W006A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F
            ADDLW   0X01
            MOVWF   ANZ_H

W006A       DECFSZ  ZAE, F
            GOTO    W006B

;--------

W007        BTFSS   TEM, BIT7
            GOTO    W008
            MOVLW   D'128'
            MOVWF   ZAE

W007B       INCF    ANZ_L, F
            MOVF    ANZ_L, W
            ANDLW   0X0F
            MOVWF   VER
            MOVLW   0X0A
            SUBWF   VER, W
            BTFSS   STATUS, Z
            MOVF    ANZ_L, W
            ANDLW   0XF0
            ADDLW   0X10
            MOVWF   ANZ_L

            MOVF    ANZ_L, W
            ANDLW   0XFF
            MOVWF   VER
            MOVLW   0XA0
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W007A
            CLRF    ANZ_L
            MOVF    ANZ_H, W
            ANDLW   0X0F
            ADDLW   0X01
            MOVWF   ANZ_H

W007A       DECFSZ  ZAE, F
            GOTO    W007B


;------------
;BCD-ANZEIGE

W008        CLRF    MERK                    ;
            CLRF    MINUS        
            BTFSS   TEMP_H, BIT7            ;BIT7  GEWHLT, ES MUSS 00HEX SEIN BEI POSITIVER TEMPERATUR
            GOTO    W111        
            BSF     MINUS, BIT0     

W111        BTFSS   IA, BIT0
            GOTO    WA2
            MOVLW   H'C9'                   ;ADRESSE 1                  
            CALL    BEFEHL
            GOTO    VB2

WA2         MOVLW   H'89'                   ;ADRESSE 1                  
            CALL    BEFEHL
VB2         MOVF    ANZ_H, W
            ANDLW   H'0F'
            ADDLW   H'30'
            MOVWF   VER
            MOVLW   0X30
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W009

            BSF     MINUS, BIT1
            MOVLW   0X20                    ;VORNULL AUSBLENDEN
            CALL    DATEN
            CLRF    VER
            BSF     VER, BIT7               ;MERKER FR NCHSTE NULL AUSBLENDEN
            GOTO    W010
            
W009        BSF     MERK, BIT0              ;
            MOVF    VER, W
            CALL    DATEN


W010        BTFSS   IA, BIT0
            GOTO    WA3
            MOVLW   H'CB'                   ;ADRESSE 3
            CALL    BEFEHL
            GOTO    VB3

WA3         MOVLW   H'8B'                   ;ADRESSE 3
            CALL    BEFEHL
VB3         MOVF    ANZ_L, W
            ANDLW   H'0F'
            ADDLW   H'30'
            CALL    DATEN

            BTFSS   IA, BIT0
            GOTO    WA4
            MOVLW   H'CA'                   ;ADRESSE 3
            CALL    BEFEHL
            GOTO    VB4

WA4         MOVLW   H'8A'                   ;ADRESSE 2
            CALL    BEFEHL
VB4         SWAPF   ANZ_L, W
            ANDLW   H'0F'
            ADDLW   H'30'
            MOVWF   VER
            MOVLW   0X30
            SUBWF   VER, W
            BTFSS   STATUS, Z
            GOTO    W011
            BTFSC   MERK, BIT0              ;
            GOTO    W011                    ;
            BSF     MINUS, BIT2
            BCF     MINUS, BIT1
            MOVLW   0X20                    ;VORNULL AUSBLENDEN
            CALL    DATEN
            GOTO    W112
            
W011        MOVF    VER, W
            CALL    DATEN

W112        BTFSS   MINUS, BIT0
            GOTO    W113
            BTFSS   MINUS, BIT2
            GOTO    W114

            BTFSS   IA, BIT0
            GOTO    WA5
            MOVLW   H'CA'                   ;ADRESSE 2
            CALL    BEFEHL
            GOTO    VB5

WA5         MOVLW   H'8A'                   ;ADRESSE 2
            CALL    BEFEHL
VB5         MOVLW   H'2D'
            CALL    DATEN

W114        BTFSS   MINUS, BIT1
            GOTO    W113

            BTFSS   IA, BIT0
            GOTO    WA6
            MOVLW   H'C9'                   ;ADRESSE 2
            CALL    BEFEHL
            GOTO    VB6

WA6         MOVLW   H'89'                   ;ADRESSE 1                  
            CALL    BEFEHL
VB6         MOVLW   H'2D'
            CALL    DATEN           


W113        RETURN

;*****************************************************************************
;*                              EEPROM WRITE                                 *
;*                              17.12.08                                     *
;*****************************************************************************
E_WRITE         BCF     PIR1, EEIF
;DATEN WELCHE GESPEICHERT WERDEN SOLLEN STEHEN HIER IN W
;ADRESSE ADR LADEN, WOHIN GESPEICHERT WERDEN SOLL
                BANK1
                MOVWF   EEDATA
                BANK0
                MOVF    ADR, W
                BANK1
                MOVWF   EEADR

                BSF     EECON1, WREN
                MOVLW   0X55
                MOVWF   EECON2
                MOVLW   0XAA
                MOVWF   EECON2
                BSF     EECON1, WR
W_1             BTFSC   EECON1, WR
                GOTO    W_1
                BSF     EECON1, WREN
                BANK0
                BCF     PIR1, EEIF

                RETURN
;DIE EEPROMADRESSE MUSS IN ADR STEHEN
;DAS ZU SPEICHERNDE BYTE IN TON

;*****************************************************************************
;*                              EEPROM READ                                  *
;*                              16.12.08                                     *
;*****************************************************************************
E_READ          BANK1
                MOVWF   EEADR
                BSF     EECON1, RD
EE1             BTFSC   EECON1, RD
                GOTO    EE1
                MOVF    EEDATA, W
                BANK0
;                MOVWF   MERK2
                RETURN
;AM ANFANG MUSS W MIT DER EEPROM-ADRESSE GELADEN SEIN
;DAS ERGEBNIS DES WERTES VOM EEPROM-INHALT STEHT AM ENDE IN W



;*****************************************************************************
;*                              INTERPOLATION DER KOMMA-STELLE               *
;*                              31.01.12                                     *
;*****************************************************************************

POLAT   MOVLW   H'06'           ;ADRESSE SPEICHERN NEUES PCLATH ALSO 0X200
        MOVWF   PCLATH

        MOVF    TEMP_L, W       
        ANDLW   0X0F            
        MOVWF   SPRUNG

        BTFSS   MERK, BIT7      ;BEI NEGATIVEN TEMPERATUREN NACH KOMMASTELLE HEX + 1BIT, AUSSER BEI 0C
        GOTO    GHG 

        COMF    SPRUNG, F
        INCF    SPRUNG, F
        MOVF    SPRUNG, W
        ANDLW   0X0F  
        MOVWF   SPRUNG

GHG     MOVF    SPRUNG, W

        ADDWF   PCL,F           ; SPRUNG AN STELLE

        RETLW   0X30            ;0 BLIND AUSGABE DA SCHON BEI 0C VORHER DARGESTELLT
        RETLW   0X31            ;1 = 0,0625
        RETLW   0X31            ;2 = 2*0,0625=0,125
        RETLW   0X32            ;3 = 3*0,0625=0,1875
        RETLW   0X33            ;4 = 4*0,0625=0,25

        RETLW   0X33            ;5
        RETLW   0X34            ;6 = 6*0,0625=0,375
        RETLW   0X34            ;7
        RETLW   0X35            ;8

        RETLW   0X36            ;9
        RETLW   0X36            ;10
        RETLW   0X37            ;11
        RETLW   0X38            ;12

        RETLW   0X38            ;13
        RETLW   0X39            ;14
        RETLW   0X39            ;15


            END





